﻿using System;
using System.Windows.Forms;
using System.Xml;
using System.Xml.Serialization;

namespace PersonXMLSerializer
{
    [XmlType(Namespace="urn:Person")]
    public class Person : IComparable<Person>, IEquatable<Person>
    {
        // member field
        protected string name;
        protected int id;
        protected string phone;
        protected string address;

        // delegate
        public static Comparison<Person> NameComparison = delegate(Person p, Person q)
        {
            return p.Name.CompareTo(q.Name);
        };

        public static Comparison<Person> IDComparison = delegate(Person p, Person q)
        {
            return p.ID.CompareTo(q.ID);
        };

        public static Comparison<Person> PhoneComparison = delegate(Person p, Person q)
        {
            return p.Phone.CompareTo(q.Phone);
        };

        public static Comparison<Person> AddressComparison = delegate(Person p, Person q)
        {
            return p.Address.CompareTo(q.Address);
        };

        [XmlAttribute(AttributeName = "Name")]
        // property
        public string Name
        {
            get
            {
                return name;
            }
            set
            {
                name = value;
            }
        }

        [XmlAttribute(AttributeName = "ID")]
        public int ID
        {
            get
            {
                return id;
            }
            set
            {
                id = value;
            }
        }

        [XmlAttribute(AttributeName = "Phone")]
        public string Phone
        {
            get
            {
                return phone;
            }
            set
            {
                phone = value;
            }
        }

        [XmlAttribute(AttributeName = "Address")]
        public string Address
        {
            get
            {
                return address;
            }
            set
            {
                address = value;
            }
        }

        // constructor
        public Person() : this("", 0, "", "")
        {
        }

        public Person(string name, int id, string phone, string address)
        {
            this.name = name;
            this.id = id;
            this.phone = phone;
            this.address = address;
        }

        public Person(Person p)
        {
            this.name = p.name;
            this.id = p.id;
            this.phone = p.phone;
            this.address = p.address;
        }

        // method
        public virtual void Print()
        {
            Console.WriteLine("이름: {0} ID: {1} 전화번호: {2} 주소: {3}", this.name, this.id, this.phone, this.address);
        }

        // needed for sort
        #region IComparable<Person> Members
        public int CompareTo(Person other)
        {
            return Name.CompareTo(other.Name);
        }
        #endregion

        // needed for search
        #region IEquatable<Person> Members
        public bool Equals(Person other)
        {
            return Name.Equals(other.Name) && ID.Equals(other.ID) && Phone.Equals(other.Phone) && Address.Equals(other.Address);
        }
        #endregion

        // System.Object.ToString method override
        public override string ToString()
        {
            return string.Format("{0},{1},{2},{3}", Name, ID, Phone, Address); // csv file format으로 출력
        }

        // needed for ListView
        #region ListViewItem
        public ListViewItem ToListViewItem()
        {
            ListViewItem item = new ListViewItem(this.name);
            item.SubItems.Add(new ListViewItem.ListViewSubItem(item, this.id.ToString()));
            item.SubItems.Add(new ListViewItem.ListViewSubItem(item, this.phone));
            item.SubItems.Add(new ListViewItem.ListViewSubItem(item, this.address));
            return item;
        }
        #endregion
    }
}
